/*----------------------------------------------------------------------------
| intent                erase.cpp/h                 bob.seabrook  September 2007
|      Erase Str71x flash bank1 sector(s) from code in bank0. 
|      (Used when reprogramming bank1)
| why
|      It is illegal to program bank1 direct from bank0 right after reset
|      [see sec 2.3-note in str71x_flash programming_reference manual.]
| how
|      This code programs bank1(sector0) from (bank0)code moved to RAM.
|      erase.cpp is a separate file so Makefile can force it to be compiled to ARM.
|      It accomodates 2 ARM functions so one can call the other directly w/o 
|      fragmentation due to interworking branches. This way the called function
|      address is always the start of a discrete unit that can be moved to RAM 
|      and run from there.
|
| assumptions
|   0) Makefile compiles this file to ARM (not Thumb) and w/o interworking.
|   1) the project never writes to flash after hard reset w/o erasing sectors
|      first,  although it is possible to use flash that way.
-----------------------------------------------------------------------------*/ 
#include <string.h> //for memcpy
#include "erase.h"    //for function ptototypes and UART_TypeDef

void erase_func(void) /*  erase_flash1() copies this to RAM on stack & runs it from there because str71x cannot 
~~~~~~~~~~~~~~~~~~~~~     program bank1 from bank0 right after a hard reset! see sec 2.3 flash prog manual - thanks Spen */
{
  	FLASHR->CR0 |=0x80000000;                  // WMS bit starts the write/erase
	
	register int timeout = 0x50000000;         //timeout needs to allow 1.1 s per 8k sector (ref sec 2.3.3 str71x_datasheet)
	while((FLASHR->CR0 & 0x14) && timeout--)   // wait for Lock and BSY1 to clear 
    {
      WDG_REFRESH;                             // WDG->KR = 0xA55A; WDG->KR = 0x5AA5;
	}             
}

u32 erase_flash1(UART_TypeDef *UARTx) /* erase flash bank1 display and progress on uart given
~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ */
{
    #define RAM_CODE_SIZE 120  // <=== examine main.lss or arm.lst to establish size of erase_func() [currently 108 bytes (decimal)]
    u8 CodeBuffer[RAM_CODE_SIZE+4];                       //allocate RAM space on stack to put flash-erase code
    typedef void (*fptr_t)(void);                         //declare a function pointer type (32bit)
    fptr_t RamCode_func = (fptr_t)CodeBuffer;             //make the func ptr point to the ram block on the stack
    memcpy(CodeBuffer,(void*)erase_func,RAM_CODE_SIZE);   //copy rom func to ram ready to run it with (*RamCode_func)();                            //jump to the loader (call actually) reset gets back here for now
  
    FLASHR->CR0 = 0;               //ensure INTM bit clear so we do NOT generate an interrupt
    FLASHR->ER = 0;                //clear any blocking errors
    FLASHR->CR0 |= 0x08000000;     //SER -- must or this as shown see examples p27/30
    FLASHR->CR1 |= 0x00010000;     //b1f0 (bank1 sector0).    0x00030000 would erase b1f1 (sector1) too.
    WDG_REFRESH;                   // WDG->KR = 0xA55A; WDG->KR = 0x5AA5;
                       
    (*RamCode_func)();             //MUST do the;  FLASH_CR0 |=0x80000000; while((FLASH_CR0 & 0x14));  from RAM!!
    return FLASHR->ER;
}
